package models

import (
	"gorm.io/datatypes"
	"gorm.io/gorm"
)

type PatientType struct {
	gorm.Model
	PatientTypeID          string
	PatientTypeName        string
	PatientTypeNameLocal   string
	PatientTypeDescription string
	Sort                   int
	IsEnabled              bool
	Product                string
	IsPreInstall           bool
}

func (PatientType) TableName() string {
	return "p_patient_type"
}

type BodyPart struct {
	gorm.Model
	BodyPartID          string
	BodyPartName        string
	BodyPartNameLocal   string
	BodyPartDescription string
	PatientType         string
	Modality            string
	Sort                int
	IsEnabled           bool
	Product             string
	IsPreInstall        bool
}

func (BodyPart) TableName() string {
	return "p_body_part"
}

type Procedure struct {
	gorm.Model
	InternalID                string `gorm:"index:,unique"`
	ProcedureID               string
	ProcedureCode             string //程序代码
	ProcedureName             string //程序名称
	ProcedureNameLocal        string
	ProcedureOtherName        string //程序别名
	ProcedureDescription      string //程序描述
	ProcedureDescriptionLocal string
	PatientType               string //病人类型
	BodyPartID                string
	ProcedureType             string  //程序类型
	FastSearch                bool    //是否快速搜索
	ProtocolLaterality        string  //协议体位
	ProcedureCategory         string  //程序类别
	Modality                  string  //设备类型
	Views                     []*View `gorm:"many2many:p_procedure_views;foreignKey:InternalID;joinForeignKey:ProcedureID;References:InternalID;joinReferences:ViewID"`
	Sort                      int
	IsEnabled                 bool
	Product                   string
	IsPreInstall              bool
}

func (Procedure) TableName() string {
	return "p_procedure"
}

type View struct {
	gorm.Model
	InternalID      string `gorm:"index:,unique"`
	ViewID          string //体位ID
	ViewName        string //体位名称
	ViewNameLocal   string
	ViewOtherName   string //体位别名
	ViewDescription string //体位描述
	ViewPosition    string //投照方向
	Application     string //RAD
	AnatomicRegion  string //解剖区域
	PatientType     string
	BodyPartID      string
	ViewIconName    string
	ViewBigIconName string
	ViewCoachName   string
	Modality        string //设备类型
	WorkStationId   int
	AprId           string //曝光参数
	ImgProcId       string //图像处理
	ConfigObject    datatypes.JSON
	Sort            int
	IsEnabled       bool
	Product         string
	IsPreInstall    bool
}

func (View) TableName() string {
	return "p_view"
}

type Apr struct {
	gorm.Model
	AprID          string `gorm:"index:,unique"`
	AprName        string
	AprDescription string
	PatientType    string
	BodyPartID     string
	ViewPosition   string
	Category       string
	Modality       string
	Sort           int
	IsEnabled      bool
	Product        string
	IsPreInstall   bool
}

func (Apr) TableName() string {
	return "p_apr"
}

type AprSub struct {
	gorm.Model
	AprID           string `gorm:"index"`
	WorkStationID   int
	PatientSize     string
	AcquisitionMode string
	ConfigObject    datatypes.JSON
	Sort            int
	IsEnabled       bool
	Product         string
	IsPreInstall    bool
}

func (AprSub) TableName() string {
	return "p_apr_sub"
}
